# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Optional, cast

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.assets.futures import forecast_get_params, forecast_get_long_term_params
from ....types.assets.futures.forecast_get_response import ForecastGetResponse
from ....types.assets.futures.forecast_get_long_term_response import ForecastGetLongTermResponse

__all__ = ["ForecastsResource", "AsyncForecastsResource"]


class ForecastsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ForecastsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#accessing-raw-response-data-eg-headers
        """
        return ForecastsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ForecastsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#with_streaming_response
        """
        return ForecastsResourceWithStreamingResponse(self)

    def get(
        self,
        symbol: str,
        *,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        estimate_uncertainty: bool | NotGiven = NOT_GIVEN,
        get_market_drivers: bool | NotGiven = NOT_GIVEN,
        get_moving_averages: bool | NotGiven = NOT_GIVEN,
        interpolate: bool | NotGiven = NOT_GIVEN,
        model_name: Optional[str] | NotGiven = NOT_GIVEN,
        price_collar_sigma: float | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ForecastGetResponse:
        """Returns a list of all forecasts made for a given future symbol.

        Forecasts are
        made at various horizons, and can be interpolated to daily values. Forecasted
        prices, estimated lower and upper bounds, and market drivers are available for
        each forecast.

        Args:
          symbol: Future symbol

          end_date: End of forecast window (YYYY-MM-DD). The returned object will contain every
              forecast made between start*date and end_date. \\__Default value* : most recent
              date with forecasts

          estimate_uncertainty: Estimate prediction uncertainty based on recent historical accuracy.

          get_market_drivers: Return market drivers for each forecast.

          get_moving_averages: Return moving averages for each forecast.

          interpolate: Interpolate between forecast horizons and return daily forecast.

          model_name: Select a specific data model to use when generating a forecast for a future
              symbol.

          price_collar_sigma: Apply an empirical price collar to the forecasts. This regulates the forecast
              when it suggests implausibly large price changes. A smaller number results in a
              more aggressive collar. Must be a positive number.

          start_date: Start of forecast window (YYYY-MM-DD). The returned object will contain every
              forecast made between start*date and end_date. \\__Default value* : most recent
              date with forecasts

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not symbol:
            raise ValueError(f"Expected a non-empty value for `symbol` but received {symbol!r}")
        return cast(
            ForecastGetResponse,
            self._get(
                f"/v1/assets/futures/forecasts/{symbol}",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    query=maybe_transform(
                        {
                            "end_date": end_date,
                            "estimate_uncertainty": estimate_uncertainty,
                            "get_market_drivers": get_market_drivers,
                            "get_moving_averages": get_moving_averages,
                            "interpolate": interpolate,
                            "model_name": model_name,
                            "price_collar_sigma": price_collar_sigma,
                            "start_date": start_date,
                        },
                        forecast_get_params.ForecastGetParams,
                    ),
                ),
                cast_to=cast(
                    Any, ForecastGetResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def get_long_term(
        self,
        symbol: str,
        *,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        horizon: Optional[int] | NotGiven = NOT_GIVEN,
        rollover_type: Optional[str] | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ForecastGetLongTermResponse:
        """
        Returns a list of booleans indicating whether the price of the given future is
        expected to increase over the next 12 months.

        Args:
          symbol: Future symbol

          end_date: End of forecast window (YYYY-MM-DD). The returned object will contain every
              forecast made between start*date and end_date. \\__Default value* : most recent
              date with forecasts

          horizon: Number of months to forecast. _Default value_ : Shortest available term for
              requested symbol.

          rollover_type: Which rollover method to use. _Default value_ : hist_vol

          start_date: Start of forecast window (YYYY-MM-DD). The returned object will contain every
              forecast made between start*date and end_date. \\__Default value* : most recent
              date with forecasts

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not symbol:
            raise ValueError(f"Expected a non-empty value for `symbol` but received {symbol!r}")
        return self._get(
            f"/v1/assets/futures/forecasts/{symbol}/long_term_forecast",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "end_date": end_date,
                        "horizon": horizon,
                        "rollover_type": rollover_type,
                        "start_date": start_date,
                    },
                    forecast_get_long_term_params.ForecastGetLongTermParams,
                ),
            ),
            cast_to=ForecastGetLongTermResponse,
        )


class AsyncForecastsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncForecastsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncForecastsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncForecastsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#with_streaming_response
        """
        return AsyncForecastsResourceWithStreamingResponse(self)

    async def get(
        self,
        symbol: str,
        *,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        estimate_uncertainty: bool | NotGiven = NOT_GIVEN,
        get_market_drivers: bool | NotGiven = NOT_GIVEN,
        get_moving_averages: bool | NotGiven = NOT_GIVEN,
        interpolate: bool | NotGiven = NOT_GIVEN,
        model_name: Optional[str] | NotGiven = NOT_GIVEN,
        price_collar_sigma: float | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ForecastGetResponse:
        """Returns a list of all forecasts made for a given future symbol.

        Forecasts are
        made at various horizons, and can be interpolated to daily values. Forecasted
        prices, estimated lower and upper bounds, and market drivers are available for
        each forecast.

        Args:
          symbol: Future symbol

          end_date: End of forecast window (YYYY-MM-DD). The returned object will contain every
              forecast made between start*date and end_date. \\__Default value* : most recent
              date with forecasts

          estimate_uncertainty: Estimate prediction uncertainty based on recent historical accuracy.

          get_market_drivers: Return market drivers for each forecast.

          get_moving_averages: Return moving averages for each forecast.

          interpolate: Interpolate between forecast horizons and return daily forecast.

          model_name: Select a specific data model to use when generating a forecast for a future
              symbol.

          price_collar_sigma: Apply an empirical price collar to the forecasts. This regulates the forecast
              when it suggests implausibly large price changes. A smaller number results in a
              more aggressive collar. Must be a positive number.

          start_date: Start of forecast window (YYYY-MM-DD). The returned object will contain every
              forecast made between start*date and end_date. \\__Default value* : most recent
              date with forecasts

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not symbol:
            raise ValueError(f"Expected a non-empty value for `symbol` but received {symbol!r}")
        return cast(
            ForecastGetResponse,
            await self._get(
                f"/v1/assets/futures/forecasts/{symbol}",
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    query=await async_maybe_transform(
                        {
                            "end_date": end_date,
                            "estimate_uncertainty": estimate_uncertainty,
                            "get_market_drivers": get_market_drivers,
                            "get_moving_averages": get_moving_averages,
                            "interpolate": interpolate,
                            "model_name": model_name,
                            "price_collar_sigma": price_collar_sigma,
                            "start_date": start_date,
                        },
                        forecast_get_params.ForecastGetParams,
                    ),
                ),
                cast_to=cast(
                    Any, ForecastGetResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def get_long_term(
        self,
        symbol: str,
        *,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        horizon: Optional[int] | NotGiven = NOT_GIVEN,
        rollover_type: Optional[str] | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ForecastGetLongTermResponse:
        """
        Returns a list of booleans indicating whether the price of the given future is
        expected to increase over the next 12 months.

        Args:
          symbol: Future symbol

          end_date: End of forecast window (YYYY-MM-DD). The returned object will contain every
              forecast made between start*date and end_date. \\__Default value* : most recent
              date with forecasts

          horizon: Number of months to forecast. _Default value_ : Shortest available term for
              requested symbol.

          rollover_type: Which rollover method to use. _Default value_ : hist_vol

          start_date: Start of forecast window (YYYY-MM-DD). The returned object will contain every
              forecast made between start*date and end_date. \\__Default value* : most recent
              date with forecasts

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not symbol:
            raise ValueError(f"Expected a non-empty value for `symbol` but received {symbol!r}")
        return await self._get(
            f"/v1/assets/futures/forecasts/{symbol}/long_term_forecast",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "end_date": end_date,
                        "horizon": horizon,
                        "rollover_type": rollover_type,
                        "start_date": start_date,
                    },
                    forecast_get_long_term_params.ForecastGetLongTermParams,
                ),
            ),
            cast_to=ForecastGetLongTermResponse,
        )


class ForecastsResourceWithRawResponse:
    def __init__(self, forecasts: ForecastsResource) -> None:
        self._forecasts = forecasts

        self.get = to_raw_response_wrapper(
            forecasts.get,
        )
        self.get_long_term = to_raw_response_wrapper(
            forecasts.get_long_term,
        )


class AsyncForecastsResourceWithRawResponse:
    def __init__(self, forecasts: AsyncForecastsResource) -> None:
        self._forecasts = forecasts

        self.get = async_to_raw_response_wrapper(
            forecasts.get,
        )
        self.get_long_term = async_to_raw_response_wrapper(
            forecasts.get_long_term,
        )


class ForecastsResourceWithStreamingResponse:
    def __init__(self, forecasts: ForecastsResource) -> None:
        self._forecasts = forecasts

        self.get = to_streamed_response_wrapper(
            forecasts.get,
        )
        self.get_long_term = to_streamed_response_wrapper(
            forecasts.get_long_term,
        )


class AsyncForecastsResourceWithStreamingResponse:
    def __init__(self, forecasts: AsyncForecastsResource) -> None:
        self._forecasts = forecasts

        self.get = async_to_streamed_response_wrapper(
            forecasts.get,
        )
        self.get_long_term = async_to_streamed_response_wrapper(
            forecasts.get_long_term,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["FutureGetHistoricalPricesParams"]


class FutureGetHistoricalPricesParams(TypedDict, total=False):
    active_contracts_only: bool
    """Return price data for currently active contracts only.

    Set to false to also retrieve price data from expired contracts.
    """

    back_adjust: bool
    """
    Back-adjust prices to account for calendar spread at contract rollover dates.
    The method used is described here:
    https://www.sierrachart.com/index.php?page=doc/ContinuousFuturesContractCharts.html#ContinuousFuturesContractDateRuleRolloverBackAdjusted
    """

    contract: Optional[str]
    """Contract year and month. _Default value_ : All available contracts"""

    end_date: Optional[str]
    """End of prices window (YYYY-MM-DD).

    The returned object will contain the entire price history for every contract
    that traded between `start_date` and `end_date`. Ignored if `contract` is
    specified. _Default value_ : most recent date with prices
    """

    rollover_method: str
    """
    The rollover date is the most recent date for which a given contract was trading
    as the front month. This parameter specifies the method used to determine the
    rollover date for contracts. Must be one of "hist_vol", "max_vol",
    "first_notice", or "last_trade" (or left blank for no rollover calculation).
    "first_notice" not available for all commodities, and defaults to "last_trade".
    """

    start_date: Optional[str]
    """Start of prices window (YYYY-MM-DD).

    The returned object will contain the entire price history for every contract
    that traded between `start_date` and `end_date`. Ignored if `contract` is
    specified. _Default value_ : most recent date with prices
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .future_list_response import FutureListResponse as FutureListResponse
from .future_get_trading_calendar_params import FutureGetTradingCalendarParams as FutureGetTradingCalendarParams
from .future_get_historical_prices_params import FutureGetHistoricalPricesParams as FutureGetHistoricalPricesParams
from .future_get_trading_calendar_response import FutureGetTradingCalendarResponse as FutureGetTradingCalendarResponse
from .future_get_historical_prices_response import (
    FutureGetHistoricalPricesResponse as FutureGetHistoricalPricesResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable, Optional

import httpx

from ..types import (
    feature_list_params,
    feature_get_weighted_index_params,
    feature_retrieve_historical_params,
    feature_transform_historical_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.transformed_feature import TransformedFeature
from ..types.feature_list_response import FeatureListResponse
from ..types.feature_retrieve_historical_response import FeatureRetrieveHistoricalResponse

__all__ = ["FeaturesResource", "AsyncFeaturesResource"]


class FeaturesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FeaturesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#accessing-raw-response-data-eg-headers
        """
        return FeaturesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FeaturesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#with_streaming_response
        """
        return FeaturesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        dataset_key: Optional[str] | NotGiven = NOT_GIVEN,
        symbol: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FeatureListResponse:
        """
        Returns the list of all available features that Hedgewise tracks or produces.
        Some of these are used to produce our price and commodity production forecasts.
        The returned features can be filtered by futures contract symbol they can relate
        or by the dataset they belong to.

        Args:
          dataset_key: Dataset key to which the features

          symbol: Futures contract symbol

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1/features",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "dataset_key": dataset_key,
                        "symbol": symbol,
                    },
                    feature_list_params.FeatureListParams,
                ),
            ),
            cast_to=FeatureListResponse,
        )

    def get_weighted_index(
        self,
        *,
        feature_codes: List[str],
        index_label: str,
        weights: Iterable[float],
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransformedFeature:
        """
        Provides a facility to create an index formed as a weighted basket of the list
        of features provided. The features provided must exist and listed as available
        at the `/v1/features` endpoint.

        Args:
          feature_codes: The list of features to include in the index

          index_label: user defined string used to name the weighted index

          weights: The list of weights to apply on the features selection to create the index

          end_date: End of transformed feature data window (YYYY-MM-DD)

          start_date: Start of transformed feature data window (YYYY-MM-DD)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1/features/weighted_index/",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "feature_codes": feature_codes,
                        "index_label": index_label,
                        "weights": weights,
                        "end_date": end_date,
                        "start_date": start_date,
                    },
                    feature_get_weighted_index_params.FeatureGetWeightedIndexParams,
                ),
            ),
            cast_to=TransformedFeature,
        )

    def retrieve_historical(
        self,
        feature_code: str,
        *,
        add_strength_for_commodity: Optional[str] | NotGiven = NOT_GIVEN,
        agg: Optional[str] | NotGiven = NOT_GIVEN,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        freq: Optional[str] | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FeatureRetrieveHistoricalResponse:
        """Returns historical values for a given feature code.

        The feature code is a unique
        identifier for a specific feature, such as weather or crop health data. Feature
        codes can be obtained with the `/v1/features` endpoint.

        Args:
          feature_code: Feature code

          add_strength_for_commodity: If a future symbol is provided and a model for that commodity exists, a signed
              strength indicator will be returned in addition to the feature value

          agg: If a `weekly` or `monthly` frequency is requested, this parameter allows to
              control how the returned data is aggregated over each period. `mean` and `last`
              are supported

          end_date: End of feature data window (YYYY-MM-DD)

          freq: By default, time-series are returned using daily time frequency. Request
              resampled data using `weekly` or `monthly` as query parameter

          start_date: Start of feature data window (YYYY-MM-DD)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not feature_code:
            raise ValueError(f"Expected a non-empty value for `feature_code` but received {feature_code!r}")
        return self._get(
            f"/v1/features/historical/{feature_code}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "add_strength_for_commodity": add_strength_for_commodity,
                        "agg": agg,
                        "end_date": end_date,
                        "freq": freq,
                        "start_date": start_date,
                    },
                    feature_retrieve_historical_params.FeatureRetrieveHistoricalParams,
                ),
            ),
            cast_to=FeatureRetrieveHistoricalResponse,
        )

    def transform_historical(
        self,
        feature_code: str,
        *,
        transform: str,
        agg: Optional[str] | NotGiven = NOT_GIVEN,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        freq: Optional[str] | NotGiven = NOT_GIVEN,
        number_of_years: int | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        window: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransformedFeature:
        """
        Provides a facility to apply transformation like computing the average of 5
        years or transpose the features time-series to create a year-on-year
        representation of the time-series of the features Feature codes can be obtained
        with the `/v1/features` endpoint.

        Args:
          feature_code: Feature code

          transform: The type of transform requested. Currently supported are `xyavg`, `rebase`,
              `zscore`, `yoy`

          agg: If a `weekly` or `monthly` frequency is requested, this parameter allows to
              control how the returned data is aggregated over each period. `mean` and `last`
              are supported

          end_date: End of transformed feature data window (YYYY-MM-DD) - not relevant for yoy

          freq: By default, time-series are returned using daily time frequency. Request
              resampled data using `weekly` or `monthly` as query parameter

          number_of_years: Number of years to perform the average on. (valid for xyavg and yoy transforms)

          start_date: Start of transformed feature data window (YYYY-MM-DD) - not relevant for yoy

          window: Number of observations used in the transformation window. (valid for xyavg and
              zscore transforms)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not feature_code:
            raise ValueError(f"Expected a non-empty value for `feature_code` but received {feature_code!r}")
        return self._get(
            f"/v1/features/transform/{feature_code}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "transform": transform,
                        "agg": agg,
                        "end_date": end_date,
                        "freq": freq,
                        "number_of_years": number_of_years,
                        "start_date": start_date,
                        "window": window,
                    },
                    feature_transform_historical_params.FeatureTransformHistoricalParams,
                ),
            ),
            cast_to=TransformedFeature,
        )


class AsyncFeaturesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFeaturesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFeaturesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFeaturesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/descarteslabs/hedgewise-sdk-python#with_streaming_response
        """
        return AsyncFeaturesResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        dataset_key: Optional[str] | NotGiven = NOT_GIVEN,
        symbol: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FeatureListResponse:
        """
        Returns the list of all available features that Hedgewise tracks or produces.
        Some of these are used to produce our price and commodity production forecasts.
        The returned features can be filtered by futures contract symbol they can relate
        or by the dataset they belong to.

        Args:
          dataset_key: Dataset key to which the features

          symbol: Futures contract symbol

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1/features",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "dataset_key": dataset_key,
                        "symbol": symbol,
                    },
                    feature_list_params.FeatureListParams,
                ),
            ),
            cast_to=FeatureListResponse,
        )

    async def get_weighted_index(
        self,
        *,
        feature_codes: List[str],
        index_label: str,
        weights: Iterable[float],
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransformedFeature:
        """
        Provides a facility to create an index formed as a weighted basket of the list
        of features provided. The features provided must exist and listed as available
        at the `/v1/features` endpoint.

        Args:
          feature_codes: The list of features to include in the index

          index_label: user defined string used to name the weighted index

          weights: The list of weights to apply on the features selection to create the index

          end_date: End of transformed feature data window (YYYY-MM-DD)

          start_date: Start of transformed feature data window (YYYY-MM-DD)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1/features/weighted_index/",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "feature_codes": feature_codes,
                        "index_label": index_label,
                        "weights": weights,
                        "end_date": end_date,
                        "start_date": start_date,
                    },
                    feature_get_weighted_index_params.FeatureGetWeightedIndexParams,
                ),
            ),
            cast_to=TransformedFeature,
        )

    async def retrieve_historical(
        self,
        feature_code: str,
        *,
        add_strength_for_commodity: Optional[str] | NotGiven = NOT_GIVEN,
        agg: Optional[str] | NotGiven = NOT_GIVEN,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        freq: Optional[str] | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> FeatureRetrieveHistoricalResponse:
        """Returns historical values for a given feature code.

        The feature code is a unique
        identifier for a specific feature, such as weather or crop health data. Feature
        codes can be obtained with the `/v1/features` endpoint.

        Args:
          feature_code: Feature code

          add_strength_for_commodity: If a future symbol is provided and a model for that commodity exists, a signed
              strength indicator will be returned in addition to the feature value

          agg: If a `weekly` or `monthly` frequency is requested, this parameter allows to
              control how the returned data is aggregated over each period. `mean` and `last`
              are supported

          end_date: End of feature data window (YYYY-MM-DD)

          freq: By default, time-series are returned using daily time frequency. Request
              resampled data using `weekly` or `monthly` as query parameter

          start_date: Start of feature data window (YYYY-MM-DD)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not feature_code:
            raise ValueError(f"Expected a non-empty value for `feature_code` but received {feature_code!r}")
        return await self._get(
            f"/v1/features/historical/{feature_code}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "add_strength_for_commodity": add_strength_for_commodity,
                        "agg": agg,
                        "end_date": end_date,
                        "freq": freq,
                        "start_date": start_date,
                    },
                    feature_retrieve_historical_params.FeatureRetrieveHistoricalParams,
                ),
            ),
            cast_to=FeatureRetrieveHistoricalResponse,
        )

    async def transform_historical(
        self,
        feature_code: str,
        *,
        transform: str,
        agg: Optional[str] | NotGiven = NOT_GIVEN,
        end_date: Optional[str] | NotGiven = NOT_GIVEN,
        freq: Optional[str] | NotGiven = NOT_GIVEN,
        number_of_years: int | NotGiven = NOT_GIVEN,
        start_date: Optional[str] | NotGiven = NOT_GIVEN,
        window: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TransformedFeature:
        """
        Provides a facility to apply transformation like computing the average of 5
        years or transpose the features time-series to create a year-on-year
        representation of the time-series of the features Feature codes can be obtained
        with the `/v1/features` endpoint.

        Args:
          feature_code: Feature code

          transform: The type of transform requested. Currently supported are `xyavg`, `rebase`,
              `zscore`, `yoy`

          agg: If a `weekly` or `monthly` frequency is requested, this parameter allows to
              control how the returned data is aggregated over each period. `mean` and `last`
              are supported

          end_date: End of transformed feature data window (YYYY-MM-DD) - not relevant for yoy

          freq: By default, time-series are returned using daily time frequency. Request
              resampled data using `weekly` or `monthly` as query parameter

          number_of_years: Number of years to perform the average on. (valid for xyavg and yoy transforms)

          start_date: Start of transformed feature data window (YYYY-MM-DD) - not relevant for yoy

          window: Number of observations used in the transformation window. (valid for xyavg and
              zscore transforms)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not feature_code:
            raise ValueError(f"Expected a non-empty value for `feature_code` but received {feature_code!r}")
        return await self._get(
            f"/v1/features/transform/{feature_code}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "transform": transform,
                        "agg": agg,
                        "end_date": end_date,
                        "freq": freq,
                        "number_of_years": number_of_years,
                        "start_date": start_date,
                        "window": window,
                    },
                    feature_transform_historical_params.FeatureTransformHistoricalParams,
                ),
            ),
            cast_to=TransformedFeature,
        )


class FeaturesResourceWithRawResponse:
    def __init__(self, features: FeaturesResource) -> None:
        self._features = features

        self.list = to_raw_response_wrapper(
            features.list,
        )
        self.get_weighted_index = to_raw_response_wrapper(
            features.get_weighted_index,
        )
        self.retrieve_historical = to_raw_response_wrapper(
            features.retrieve_historical,
        )
        self.transform_historical = to_raw_response_wrapper(
            features.transform_historical,
        )


class AsyncFeaturesResourceWithRawResponse:
    def __init__(self, features: AsyncFeaturesResource) -> None:
        self._features = features

        self.list = async_to_raw_response_wrapper(
            features.list,
        )
        self.get_weighted_index = async_to_raw_response_wrapper(
            features.get_weighted_index,
        )
        self.retrieve_historical = async_to_raw_response_wrapper(
            features.retrieve_historical,
        )
        self.transform_historical = async_to_raw_response_wrapper(
            features.transform_historical,
        )


class FeaturesResourceWithStreamingResponse:
    def __init__(self, features: FeaturesResource) -> None:
        self._features = features

        self.list = to_streamed_response_wrapper(
            features.list,
        )
        self.get_weighted_index = to_streamed_response_wrapper(
            features.get_weighted_index,
        )
        self.retrieve_historical = to_streamed_response_wrapper(
            features.retrieve_historical,
        )
        self.transform_historical = to_streamed_response_wrapper(
            features.transform_historical,
        )


class AsyncFeaturesResourceWithStreamingResponse:
    def __init__(self, features: AsyncFeaturesResource) -> None:
        self._features = features

        self.list = async_to_streamed_response_wrapper(
            features.list,
        )
        self.get_weighted_index = async_to_streamed_response_wrapper(
            features.get_weighted_index,
        )
        self.retrieve_historical = async_to_streamed_response_wrapper(
            features.retrieve_historical,
        )
        self.transform_historical = async_to_streamed_response_wrapper(
            features.transform_historical,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .asset import Asset as Asset
from .forecast_get_params import ForecastGetParams as ForecastGetParams
from .forecast_get_response import ForecastGetResponse as ForecastGetResponse
from .indicator_list_response import IndicatorListResponse as IndicatorListResponse
from .indicator_get_hedge_params import IndicatorGetHedgeParams as IndicatorGetHedgeParams
from .indicator_get_hedge_response import IndicatorGetHedgeResponse as IndicatorGetHedgeResponse
from .forecast_get_long_term_params import ForecastGetLongTermParams as ForecastGetLongTermParams
from .forecast_get_long_term_response import ForecastGetLongTermResponse as ForecastGetLongTermResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .assets import (
    AssetsResource,
    AsyncAssetsResource,
    AssetsResourceWithRawResponse,
    AsyncAssetsResourceWithRawResponse,
    AssetsResourceWithStreamingResponse,
    AsyncAssetsResourceWithStreamingResponse,
)
from .futures import (
    FuturesResource,
    AsyncFuturesResource,
    FuturesResourceWithRawResponse,
    AsyncFuturesResourceWithRawResponse,
    FuturesResourceWithStreamingResponse,
    AsyncFuturesResourceWithStreamingResponse,
)

__all__ = [
    "FuturesResource",
    "AsyncFuturesResource",
    "FuturesResourceWithRawResponse",
    "AsyncFuturesResourceWithRawResponse",
    "FuturesResourceWithStreamingResponse",
    "AsyncFuturesResourceWithStreamingResponse",
    "AssetsResource",
    "AsyncAssetsResource",
    "AssetsResourceWithRawResponse",
    "AsyncAssetsResourceWithRawResponse",
    "AssetsResourceWithStreamingResponse",
    "AsyncAssetsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .performance_list_response import PerformanceListResponse as PerformanceListResponse
from .performance_retrieve_params import PerformanceRetrieveParams as PerformanceRetrieveParams
from .performance_retrieve_response import PerformanceRetrieveResponse as PerformanceRetrieveResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from .asset import Asset
from ...._models import BaseModel

__all__ = ["IndicatorListResponse", "Data"]


class Data(BaseModel):
    assets: List[Asset]

    name: str


class IndicatorListResponse(BaseModel):
    data: List[Data]

    success: Literal[True]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from hedgewise import Hedgewise, AsyncHedgewise
from tests.utils import assert_matches_type
from hedgewise.types import ForexListResponse, ForexRetrieveResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestForex:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve(self, client: Hedgewise) -> None:
        forex = client.forex.retrieve(
            code="EUR",
        )
        assert_matches_type(ForexRetrieveResponse, forex, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_retrieve_with_all_params(self, client: Hedgewise) -> None:
        forex = client.forex.retrieve(
            code="EUR",
            end_date="2025-04-25",
            foreign_per_usd=True,
            start_date="2025-04-17",
        )
        assert_matches_type(ForexRetrieveResponse, forex, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_retrieve(self, client: Hedgewise) -> None:
        response = client.forex.with_raw_response.retrieve(
            code="EUR",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        forex = response.parse()
        assert_matches_type(ForexRetrieveResponse, forex, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_retrieve(self, client: Hedgewise) -> None:
        with client.forex.with_streaming_response.retrieve(
            code="EUR",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            forex = response.parse()
            assert_matches_type(ForexRetrieveResponse, forex, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_retrieve(self, client: Hedgewise) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `code` but received ''"):
            client.forex.with_raw_response.retrieve(
                code="",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_list(self, client: Hedgewise) -> None:
        forex = client.forex.list()
        assert_matches_type(ForexListResponse, forex, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_list(self, client: Hedgewise) -> None:
        response = client.forex.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        forex = response.parse()
        assert_matches_type(ForexListResponse, forex, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_list(self, client: Hedgewise) -> None:
        with client.forex.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            forex = response.parse()
            assert_matches_type(ForexListResponse, forex, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncForex:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncHedgewise) -> None:
        forex = await async_client.forex.retrieve(
            code="EUR",
        )
        assert_matches_type(ForexRetrieveResponse, forex, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncHedgewise) -> None:
        forex = await async_client.forex.retrieve(
            code="EUR",
            end_date="2025-04-25",
            foreign_per_usd=True,
            start_date="2025-04-17",
        )
        assert_matches_type(ForexRetrieveResponse, forex, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncHedgewise) -> None:
        response = await async_client.forex.with_raw_response.retrieve(
            code="EUR",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        forex = await response.parse()
        assert_matches_type(ForexRetrieveResponse, forex, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncHedgewise) -> None:
        async with async_client.forex.with_streaming_response.retrieve(
            code="EUR",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            forex = await response.parse()
            assert_matches_type(ForexRetrieveResponse, forex, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncHedgewise) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `code` but received ''"):
            await async_client.forex.with_raw_response.retrieve(
                code="",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_list(self, async_client: AsyncHedgewise) -> None:
        forex = await async_client.forex.list()
        assert_matches_type(ForexListResponse, forex, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncHedgewise) -> None:
        response = await async_client.forex.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        forex = await response.parse()
        assert_matches_type(ForexListResponse, forex, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncHedgewise) -> None:
        async with async_client.forex.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            forex = await response.parse()
            assert_matches_type(ForexListResponse, forex, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["FeatureListResponse", "Data"]


class Data(BaseModel):
    feature_code: str

    long_name: str

    type: str

    variable_type: str

    country: Optional[str] = None

    datasets: Optional[List[str]] = None

    frequency: Optional[str] = None

    horizons: Optional[List[str]] = None

    location: Optional[str] = None

    main_commodity: Optional[str] = None

    phenology_stage: Optional[List[str]] = None

    recent_influence: Optional[float] = None

    source: Optional[str] = None

    statistic_type: Optional[str] = None

    symbols: Optional[List[str]] = None

    unit: Optional[str] = None


class FeatureListResponse(BaseModel):
    data: List[Data]

    success: Literal[True]
